@@ -3,10 +3,10 @@
 ;; Copyright (C) 2011-2017 Ryan C. Thompson
 
 ;; Filename: ido-completing-read+.el
-;; Author: Ryan Thompson
+;; Author: Ryan C. Thompson <rct@thompsonclan.org>
 ;; Created: Sat Apr  4 13:41:20 2015 (-0700)
-;; Version: 4.13
-;; Package-Requires: ((emacs "24.4") (cl-lib "0.5") (s "0.1") (memoize "1.1"))
+;; Version: 4.14
+;; Package-Requires: ((emacs "24.4") (seq "0.5") (memoize "1.1"))
 ;; URL: https://github.com/DarwinAwardWinner/ido-completing-read-plus
 ;; Keywords: ido, completion, convenience
 
@@ -48,7 +48,7 @@
 ;; this mode is enabled. Some other functions have ido disabled in
 ;; them because their packages already provide support for ido via
 ;; other means (for example, magit). See `M-x describe-variable
-;; ido-cr+-function-blacklist' for more information.
+;; ido-cr+-disable-list' for more information.
 
 ;; ido-completing-read+ version 4.0 is a major update. The formerly
 ;; separate package ido-ubiquitous has been subsumed into
@@ -77,17 +77,18 @@
 ;;
 ;;; Code:
 
-(defconst ido-completing-read+-version "4.13"
+(defconst ido-completing-read+-version "4.14"
   "Currently running version of ido-completing-read+.
 
 Note that when you update ido-completing-read+, this variable may
 not be updated until you restart Emacs.")
 
+(require 'nadvice)
 (require 'ido)
+(require 'seq)
 (require 'minibuf-eldef)
 (require 'cl-lib)
 (require 'cus-edit)
-(require 's)
 
 ;; Optional dependency, only needed for optimization
 (require 'memoize nil t)
@@ -103,11 +104,15 @@ not be updated until you restart Emacs.")
   "If non-nil, ido-cr+ will print debug info.
 
 Debug info is printed to the *Messages* buffer."
-  nil
+  :init-value nil
   :global t
   :group 'ido-completing-read-plus)
 
 (defsubst ido-cr+--debug-message (format-string &rest args)
+  "Emit a debug message for ido-cr+.
+
+This only has an effect when `ido-cr+-debug-mode' is non-nil.
+Arguments are identical to `message'."
   (when ido-cr+-debug-mode
     (apply #'message (concat "ido-completing-read+: " format-string) args)))
 
@@ -117,8 +122,8 @@ Debug info is printed to the *Messages* buffer."
 ;; silence byte-compiler warnings, despite already being declared in
 ;; ido.el.
 
-(defmacro define-ido-internal-var (symbol &optional initvalue docstring)
-  "Declare and initialize an ido internal variable.
+(defmacro ido-cr+-define-ido-internal-var (symbol &optional initvalue docstring)
+  "Declare and initialize SYMBOL an ido internal variable.
 
 This is used to suppress byte-compilation warnings about
 reference to free variables when ido-cr+ attempts to access
@@ -139,11 +144,11 @@ package's variable is not safe in general, but in this case it
 should be, because ido always let-binds this variable before
 using it, so the initial value shouldn't matter.")))
 
-(define-ido-internal-var ido-context-switch-command)
-(define-ido-internal-var ido-cur-list)
-(define-ido-internal-var ido-cur-item)
-(define-ido-internal-var ido-require-match)
-(define-ido-internal-var ido-process-ignore-lists)
+(ido-cr+-define-ido-internal-var ido-context-switch-command)
+(ido-cr+-define-ido-internal-var ido-cur-list)
+(ido-cr+-define-ido-internal-var ido-cur-item)
+(ido-cr+-define-ido-internal-var ido-require-match)
+(ido-cr+-define-ido-internal-var ido-process-ignore-lists)
 
 ;; Vars and functions from flx-ido package
 (defvar flx-ido-mode)
@@ -154,7 +159,7 @@ using it, so the initial value shouldn't matter.")))
   "Minibuffer depth of the most recent ido-cr+ activation.
 
 If this equals the current minibuffer depth, then the minibuffer
-is currently being used by ido-cr+, and ido-cr+ feature will be
+is currently being used by ido-cr+, and ido-cr+ features will be
 active. Otherwise, something else is using the minibuffer and
 ido-cr+ features will be deactivated to avoid interfering with
 the other command.
@@ -194,7 +199,7 @@ dynamically.")
   "Idle timer for updating dynamic completion list.")
 
 (defvar ido-cr+-exhibit-pending nil
-  "This is non-nil after calling `ido-tidy' until the next call to `ido-exhibit'.
+  "This is non-nil between calling `ido-tidy' and `ido-exhibit'.
 
 Typically this is non-nil while any command is running and nil at all
 other times, since those two functions are in `pre-command-hook'
@@ -230,6 +235,30 @@ Each element is a cons cell of (REMOVEP . TEXT), where REMOVEP is
 the prefix argument to `ido-restrict-to-matches' and TEXT is the
 pattern used to restrict.")
 
+(defvar ido-cr+-need-bug27807-workaround
+  (cl-letf*
+      ((ido-exit ido-exit)
+       ((symbol-function 'read-from-minibuffer)
+        (lambda (_prompt &optional initial-contents &rest _remaining-args)
+          (setq ido-exit 'takeprompt) ; Emulate pressing C-j in ido
+          (if (consp initial-contents)
+              (substring (car initial-contents) 0 (1- (cdr initial-contents)))
+            initial-contents)))
+       ;; Need to get the unadvised original of `ido-completing-read'
+       ;; because the advice is autoloaded, so calling it while
+       ;; loading the package will trigger a recursive load.
+       ((symbol-function 'ido-completing-read)
+        (advice--cd*r (symbol-function 'ido-completing-read)))
+       (input-before-point
+        (ido-completing-read "Pick: " '("aaa" "aab" "aac") nil nil '("aa" . 1))))
+    ;; If an initial position of 1 yields a 0-length string, then this
+    ;; Emacs does not have the bug fix and requires the workaround.
+    (= (length input-before-point) 0))
+  "If non-nil, enable the workaround for Emacs bug #27807.
+
+This variable is normally set when ido-cr+ is loaded, and should
+not need to be modified by users.")
+
 (defgroup ido-completing-read-plus nil
   "Extra features and compatibility for `ido-completing-read'."
   :group 'ido)
@@ -248,7 +277,7 @@ pattern used to restrict.")
               ido-ubiquitous-completing-read))
       'completing-read-default
     completing-read-function)
-  "Alternate completing-read function to use when ido is not wanted.
+  "Alternate `completing-read-function' to use when ido is not wanted.
 
 This will be used for functions that are incompatible with ido
 or if ido cannot handle the completion arguments. It will also be
@@ -278,9 +307,15 @@ disable fallback based on collection size, set this to nil."
                         widget)))))
   :group 'ido-completing-read-plus)
 
-(defcustom ido-cr+-function-blacklist
+(define-obsolete-variable-alias
+  'ido-cr+-function-blacklist
+  'ido-cr+-disable-list
+  "ido-completing-read+ 4.14")
+
+(defcustom ido-cr+-disable-list
   '(read-file-name-internal
     read-buffer
+    internal-complete-buffer
     ;; https://github.com/DarwinAwardWinner/ido-completing-read-plus/issues/60
     todo-add-category
     ;; Gnus already supports ido on its own
@@ -302,6 +337,7 @@ disable fallback based on collection size, set this to nil."
     ffap-read-file-or-url-internal
     ;; https://github.com/DarwinAwardWinner/ido-completing-read-plus/issues/161
     sly-read-symbol-name
+    org-olpath-completing-read
     )
   "Functions & commands for which ido-cr+ should be disabled.
 
@@ -310,10 +346,10 @@ fall back specifically for the named function. A regular
 expression means to fall back for any function whose name matches
 that regular expression. When ido-cr+ is called through
 `completing-read', if any function in the call stack of the
-current command matches any of the blacklist entries, ido-cr+
+current command matches any of the disable list entries, ido-cr+
 will be disabled for that command. Additionally, if the
-collection in the call to `completing-read' matches any of the
-blacklist entries, ido-cr+ will be disabled.
+collection in the call to `completing-read' is a function name
+that matches any of the entries, ido-cr+ will be disabled.
 
 Note that using specific function names is generally preferable
 to regular expressions, because the associated function
@@ -324,23 +360,45 @@ regular expressions, only name-based matching is possible."
   :type '(repeat (choice (symbol :tag "Function or command name")
                          (string :tag "Regexp"))))
 
-(defcustom ido-cr+-function-whitelist
+(define-obsolete-variable-alias
+  'ido-cr+-function-whitelist
+  'ido-cr+-allow-list
+  "ido-completing-read+ 4.14")
+
+(defcustom ido-cr+-allow-list
   nil
-  "Functions & commands for which ido-cr+ should be enabled.
+  "If non-nil, limit ido-cr+ only to the specified commands & functions.
 
-If this variable is nil, the whitelist will not be used, and
-ido-cr+ will be allowed in all functions/commands not listed in
-`ido-cr+-function-backlist'.
+If this variable is nil, the ido-cr+ will be enabled for all
+commands and functions not specified in all commands/functions
+not specified in `ido-cr+-function-backlist'.
 
-If this variable is non-nil, ido-cr+'s whitelisting mode will be
+If this variable is non-nil, ido-cr+'s limited mode will be
 enabled, and ido-cr+ will be disabled for *all* functions unless
-they match one of the entries. Matching is done in the same
-manner as `ido-cr+-function-blacklist', and blacklisting takes
-precedence over whitelisting."
+they match one of the entries in this variable. Matching is done
+in the same manner as `ido-cr+-disable-list', and the disable
+list also takes precedence over the allow list."
   :group 'ido-completing-read-plus
   :type '(repeat (choice (symbol :tag "Function or command name")
                          (string :tag "Regexp"))))
 
+(defvaralias 'ido-cr+-nil-def-wall-of-shame
+  'ido-cr+-nil-def-alternate-behavior-list
+  "Functions and commands that use `completing-read' improperly.
+
+Many functions that call `completing-read' are written with the
+assumption that the setting the REQUIRE-MATCH argument of
+`completing-read' to t means it is required to return a match.
+While that would make logical sense, it's wrong. the docstring
+for `completing-read' describes the correct behavior.
+
+> If the input is null, ‘completing-read’ returns DEF, or the
+> first element of the list of default values, or an empty string
+> if DEF is nil, regardless of the value of REQUIRE-MATCH.
+
+This can be avoided by passing an element of COLLECTION as DEF
+instead of leaving it as nil.")
+
 (defcustom ido-cr+-nil-def-alternate-behavior-list
   '("\\`describe-\\(function\\|variable\\)\\'"
     "\\`wl-"
@@ -363,36 +421,20 @@ precedence over whitelisting."
     )
   "Functions & commands with alternate behavior when DEF is nil.
 
-This variable has the same format as
-`ido-cr+-function-blacklist'. When `ido-completing-read+` is
-called through `completing-read' by/with any command, function,
-or collection matched by entries in this list, it will behave
-differently when DEF is nil. Instead of using the empty string as
-the default value, it will use the first element of COLLECTION.
+This variable has the same format as `ido-cr+-disable-list'. When
+`ido-completing-read+` is called through `completing-read'
+by/with any command, function, or collection matched by entries
+in this list, it will behave differently when DEF is nil. Instead
+of using the empty string as the default value, it will use the
+first element of COLLECTION.
 
 This is needed for optimal compatibility with commands written
-under the assumption that REQUIRE-MATCH means that a match is
-required."
+under the reasonable but wrong assumption that REQUIRE-MATCH
+means that a match is required."
   :group 'ido-completing-read-plus
   :type '(repeat (choice (symbol :tag "Function or command name")
                          (string :tag "Regexp"))))
 
-(defvaralias 'ido-cr+-nil-def-wall-of-shame 'ido-cr+-nil-def-alternate-behavior-list
-  "Functions and commands whose authors need to read the docstring for `completing-read'.
-
-Many functions that call `completing-read' are written with the
-assumption that the setting the REQUIRE-MATCH argument of
-`completing-read' to t means it is required to return a match.
-While that would make logical sense, it's wrong. the docstring
-for `completing-read' describes the correct behavior.
-
-> If the input is null, ‘completing-read’ returns DEF, or the
-> first element of the list of default values, or an empty string
-> if DEF is nil, regardless of the value of REQUIRE-MATCH.
-
-This can be avoided by passing an element of COLLECTION as DEF
-instead of leaving it as nil.")
-
 ;;;###autoload
 (defcustom ido-cr+-replace-completely nil
   "If non-nil, replace `ido-completeing-read' completely with ido-cr+.
@@ -407,35 +449,43 @@ https://github.com/DarwinAwardWinner/ido-completing-read-plus/issues"
 (define-error 'ido-cr+-fallback "ido-cr+-fallback")
 
 (defsubst ido-cr+--explain-fallback (arg)
-  ;; This function accepts a string, or an ido-cr+-fallback
-  ;; signal.
+  "Emit a debug message explaining the reason for falling back.
+
+ARG can be a string or an ido-cr+-fallback signal. In the latter
+case, the DATA part of the signal is used as the message."
   (when ido-cr+-debug-mode
     (when (and (listp arg)
                (eq (car arg) 'ido-cr+-fallback))
       (setq arg (cadr arg)))
-    (ido-cr+--debug-message "Falling back to `%s' because %s."
-                            ido-cr+-fallback-function arg)))
+    (ido-cr+--debug-message
+     "Falling back to `%s' because %s."
+     (if (symbolp ido-cr+-fallback-function)
+         ido-cr+-fallback-function
+       "ido-cr+-fallback-function")
+     arg)))
 
 ;;;###autoload
 (defsubst ido-cr+-active ()
-  "Returns non-nil if ido-cr+ is currently using the minibuffer."
+  "Return non-nil if ido-cr+ is currently using the minibuffer."
   (>= ido-cr+-minibuffer-depth (minibuffer-depth)))
 
 (defun ido-cr+--called-from-completing-read ()
-  "Returns non-nil if the most recent call to ido-cr+ was from `completing-read'."
+  "Return non-nil if the most recent call to ido-cr+ was from `completing-read'."
   (equal (cadr (backtrace-frame 1 'ido-completing-read+))
          'completing-read))
 
 (defmacro ido-cr+-function-is-in-list (fun fun-list &optional list-name)
   "Return non-nil if FUN matches an entry in FUN-LIST.
 
-This is used to check for matches to `ido-cr+-function-blacklist'
-and `ido-cr+-function-whitelist'. Read those docstrings to see
-how the matching is done.
+This is used to check for matches to `ido-cr+-disable-list' and
+`ido-cr+-allow-list'. Read those docstrings to see how
+the matching is done.
 
 This is declared as macro only in order to extract the variable
 name used for the second argument so it can be used in a debug
-message. It should be called as if it were a normal function."
+message. It should be called as if it were a normal function. The
+optional 3rd argument LIST-NAME can be used to provide this
+information manually if it is known."
   (when (null list-name)
     (if (symbolp fun-list)
         (setq list-name (symbol-name fun-list))
@@ -461,39 +511,51 @@ message. It should be called as if it were a normal function."
         ((stringp entry)
          (and (symbolp ,fun)
               (string-match-p entry (symbol-name ,fun))))
-        ;; Anything else: invalid blacklist entry
+        ;; Anything else: invalid list entry
         (t
          (ido-cr+--debug-message "Ignoring invalid entry in %s: `%S'" ,list-name entry)
          nil))
     return entry
-    ;; If no blacklist entry matches, return nil
+    ;; If no list entry matches, return nil
     finally return nil))
 
-(defun ido-cr+-function-is-blacklisted (fun)
-  "Return non-nil if FUN is blacklisted.
+(define-obsolete-function-alias
+  'ido-cr+-function-is-blacklisted
+  'ido-cr+-disabled-in-function-p
+  "ido-completing-read+ 4.14")
 
-See `ido-cr+-function-blacklist'."
-  (ido-cr+-function-is-in-list fun ido-cr+-function-blacklist))
+(defsubst ido-cr+-disabled-in-function-p (fun)
+  "Return non-nil if ido-cr+ is disabled for FUN.
 
-(defun ido-cr+-function-is-whitelisted (fun)
-  "Return non-nil if FUN is whitelisted.
+See `ido-cr+-disable-list'."
+  (ido-cr+-function-is-in-list fun ido-cr+-disable-list))
 
-See `ido-cr+-function-whitelist'."
-  (or (null ido-cr+-function-whitelist)
-      (ido-cr+-function-is-in-list fun ido-cr+-function-whitelist)))
+(define-obsolete-function-alias
+  'ido-cr+-function-is-whitelisted
+  'ido-cr+-allowed-in-function-p
+  "ido-completing-read+ 4.14")
+
+(defsubst ido-cr+-allowed-in-function-p (fun)
+  "Return non-nil if ido-cr+ is allowed for FUN.
+
+See `ido-cr+-allow-list'."
+  (or (null ido-cr+-allow-list)
+      (ido-cr+-function-is-in-list fun ido-cr+-allow-list)))
 
 ;;;###autoload
 (defun ido-completing-read+ (prompt collection &optional predicate
                                     require-match initial-input
                                     hist def inherit-input-method)
-  "ido-based method for reading from the minibuffer with completion.
+  "Ido-based method for reading from the minibuffer with completion.
 
 See `completing-read' for the meaning of the arguments.
 
 This function is a wrapper for `ido-completing-read' designed to
 be used as the value of `completing-read-function'. Importantly,
 it detects edge cases that ido cannot handle and uses normal
-completion for them."
+completion for them.
+
+See `completing-read' for the meaning of the arguments."
   (let* (;; Save the original arguments in case we need to do the
          ;; fallback
          (ido-cr+-orig-completing-read-args
@@ -534,8 +596,8 @@ completion for them."
           (if (and ido-cr+-dynamic-collection (featurep 'memoize))
               (memoize (indirect-function 'all-completions))
             'all-completions))
-         ;; If the whitelist is empty, everything is whitelisted
-         (whitelisted (not ido-cr+-function-whitelist))
+         ;; If the allow list is empty, everything is allowed
+         (ido-cr+-allowed (not ido-cr+-allow-list))
          ;; If non-nil, we need alternate nil DEF handling
          (alt-nil-def nil))
     (condition-case sig
@@ -545,23 +607,23 @@ completion for them."
             (signal 'ido-cr+-fallback
                     '("ido cannot handle alternate input methods")))
 
-          ;; Check for black/white-listed collection function
+          ;; Check for allow/disable-listed collection function
           (when (functionp collection)
-            ;; Blacklist
-            (when (ido-cr+-function-is-blacklisted collection)
+            ;; Disable list
+            (when (ido-cr+-disabled-in-function-p collection)
               (if (symbolp collection)
                   (signal 'ido-cr+-fallback
-                          (list (format "collection function `%S' is blacklisted" collection)))
+                          (list (format "collection function `%S' is disabled" collection)))
                 (signal 'ido-cr+-fallback
-                        (list "collection function is blacklisted"))))
-            ;; Whitelist
-            (when (and (not whitelisted)
-                       (ido-cr+-function-is-whitelisted collection))
+                        (list "collection function is disabled"))))
+            ;; Allow list
+            (when (and (not ido-cr+-allowed)
+                       (ido-cr+-allowed-in-function-p collection))
               (ido-cr+--debug-message
                (if (symbolp collection)
-                   (format "Collection function `%S' is whitelisted" collection)
-                 "Collection function is whitelisted"))
-              (setq whitelisted t))
+                   (format "Collection function `%S' is allowed" collection)
+                 "Collection function is allowed"))
+              (setq ido-cr+-allowed t))
             ;; nil DEF list
             (when (and
                    require-match (null def)
@@ -594,21 +656,21 @@ completion for them."
                       ido-cr+-max-items))))
 
           ;; If called from `completing-read', check for
-          ;; black/white-listed commands/callers
+          ;; disabled/allowed commands/callers
           (when (ido-cr+--called-from-completing-read)
             ;; Check calling command and `ido-cr+-current-command'
             (cl-loop
              for cmd in (list this-command ido-cr+-current-command)
 
-             if (ido-cr+-function-is-blacklisted cmd)
+             if (ido-cr+-disabled-in-function-p cmd)
              do (signal 'ido-cr+-fallback
-                        (list "calling command `%S' is blacklisted" cmd))
+                        (list "calling command `%S' is disabled" cmd))
 
-             if (and (not whitelisted)
-                     (ido-cr+-function-is-whitelisted cmd))
+             if (and (not ido-cr+-allowed)
+                     (ido-cr+-allowed-in-function-p cmd))
              do (progn
-                  (ido-cr+--debug-message "Command `%S' is whitelisted" cmd)
-                  (setq whitelisted t))
+                  (ido-cr+--debug-message "Command `%S' is allowed" cmd)
+                  (setq ido-cr+-allowed t))
 
              if (and
                  require-match (null def) (not alt-nil-def)
@@ -633,20 +695,20 @@ completion for them."
                                       '(internal--funcall-interactively
                                         (indirect-function 'call-interactively))))
 
-                     if (ido-cr+-function-is-blacklisted caller)
+                     if (ido-cr+-disabled-in-function-p caller)
                      do (signal 'ido-cr+-fallback
                                 (list (if (symbolp caller)
-                                          (format "calling function `%S' is blacklisted" caller)
-                                        "a calling function is blacklisted")))
+                                          (format "calling function `%S' is disabled" caller)
+                                        "a calling function is disabled")))
 
-                     if (and (not whitelisted)
-                             (ido-cr+-function-is-whitelisted caller))
+                     if (and (not ido-cr+-allowed)
+                             (ido-cr+-allowed-in-function-p caller))
                      do (progn
                           (ido-cr+--debug-message
                            (if (symbolp caller)
-                               (format "Calling function `%S' is whitelisted" caller)
-                             "A calling function is whitelisted"))
-                          (setq whitelisted t))
+                               (format "Calling function `%S' is allowed" caller)
+                             "A calling function is allowed"))
+                          (setq ido-cr+-allowed t))
 
                      if (and require-match (null def) (not alt-nil-def)
                              (ido-cr+-function-is-in-list
@@ -658,9 +720,9 @@ completion for them."
                              "Using alternate nil DEF handling for a calling function"))
                           (setq alt-nil-def t))))
 
-          (unless whitelisted
+          (unless ido-cr+-allowed
             (signal 'ido-cr+-fallback
-                    (list "no functions or commands matched the whitelist for this call")))
+                    (list "no functions or commands matched the allow list for this call")))
 
           (when (and require-match (null def))
             ;; Replace nil with "" for DEF if match is required, unless
@@ -696,9 +758,9 @@ completion for them."
                     '("ido cannot handle the empty string as an option when `ido-enable-dot-prefix' is non-nil; see https://debbugs.gnu.org/cgi/bugreport.cgi?bug=26997")))
 
           ;; Fix ido's broken handling of cons-style INITIAL-INPUT on
-          ;; Emacsen older than 27.1.
+          ;; Emacsen older than 27. See Emacs bug #27807.
           (when (and (consp initial-input)
-                     (version< emacs-version "27.1"))
+                     ido-cr+-need-bug27807-workaround)
             ;; `completing-read' uses 0-based index while
             ;; `read-from-minibuffer' uses 1-based index.
             (cl-incf (cdr initial-input)))
@@ -813,10 +875,12 @@ called through ido-cr+."
             #'ido-select-text@ido-cr+-fix-require-match)
 
 (defun ido-tidy@ido-cr+-set-exhibit-pending (&rest _args)
+  "Advice to manage the value of `ido-cr+-exhibit-pending'."
   (setq ido-cr+-exhibit-pending t))
 (advice-add 'ido-tidy :after 'ido-tidy@ido-cr+-set-exhibit-pending)
 
 (defun ido-exhibit@ido-cr+-clear-exhibit-pending (&rest _args)
+  "Advice to manage the value of `ido-cr+-exhibit-pending'."
   (setq ido-cr+-exhibit-pending nil))
 (advice-add 'ido-exhibit :before 'ido-exhibit@ido-cr+-clear-exhibit-pending)
 
@@ -843,7 +907,7 @@ not a function, this is equivalent to
      for i from 0 upto (length string)
      append (funcall
              ido-cr+-all-completions-memoized
-             (s-left i string)
+             (substring string 0 i)
              collection
              predicate)
      into completion-list
@@ -888,7 +952,7 @@ result."
      filtered-collection)))
 
 (defun ido-cr+-cyclicp (x)
-  "Returns non-nill if X is a list containing a circular reference."
+  "Return non-nill if X is a list containing a circular reference."
   (cl-loop
    for tortoise on x
    for hare on (cdr x) by #'cddr
@@ -908,7 +972,7 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
       ;; If current `ido-text' is equal to or a prefix of the previous
       ;; one, a dynamic update is not needed.
       (when (or (null ido-cr+-last-dynamic-update-text)
-                (not (s-prefix? ido-text ido-cr+-last-dynamic-update-text)))
+                (not (string-prefix-p ido-text ido-cr+-last-dynamic-update-text)))
         (ido-cr+--debug-message "Doing a dynamic update because `ido-text' changed from %S to %S"
                                 ido-cr+-last-dynamic-update-text ido-text)
         (setq ido-cr+-last-dynamic-update-text ido-text)
@@ -923,7 +987,7 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
                      ;; If `ido-text' is a prefix of `first-match', then we
                      ;; only need to check `first-match'
                      ((and first-match
-                           (s-prefix? ido-text first-match))
+                           (string-prefix-p ido-text first-match))
                       (list first-match))
                      ;; Otherwise we need to check both
                      (t
@@ -1003,7 +1067,7 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
                                  #'ido-cr+-update-dynamic-collection)))))
 
 (defun ido-cr+-minibuffer-setup ()
-  "set up minibuffer `post-command-hook' for ido-cr+ "
+  "Set up minibuffer `post-command-hook' for ido-cr+."
   (when (ido-cr+-active)
     (add-hook 'post-command-hook
               'ido-cr+-schedule-dynamic-collection-update)))
@@ -1035,7 +1099,7 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
 ;; dynamically-added completions are also properly restricted.
 (defun ido-restrict-to-matches@ido-cr+-record-restriction
     (&optional removep)
-  "Record the restriction criterion for ido-cr+"
+  "Record the restriction criterion for ido-cr+."
   (ido-cr+--debug-message "Appending restriction %S to `ido-cr+-active-restrictions'"
                           (cons removep ido-text))
   (add-to-list 'ido-cr+-active-restrictions (cons removep ido-text) t))
@@ -1046,7 +1110,7 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
 ;; default when the input is empty and the empty string is the
 ;; selected choice
 (defun minibuf-eldef-update-minibuffer@ido-cr+-compat (orig-fun &rest args)
-  "This advice allows minibuffer-electric-default-mode to work with ido-cr+."
+  "This advice allows `minibuffer-electric-default-mode' to work with ido-cr+."
   (if (ido-cr+-active)
       (unless (eq minibuf-eldef-showing-default-in-prompt
                   (and (string= (car ido-cur-list) "")
@@ -1066,8 +1130,8 @@ This has no effect unless `ido-cr+-dynamic-collection' is non-nil."
 
 If this mode causes problems for a function, you can customize
 when ido completion is or is not used by customizing
-`ido-cr+-function-blacklist'."
-  nil
+`ido-cr+-disable-list'."
+  :init-value nil
   :global t
   :group 'ido-completing-read-plus
   ;; Actually enable/disable the mode by setting
@@ -1077,37 +1141,35 @@ when ido completion is or is not used by customizing
             #'ido-completing-read+
           ido-cr+-fallback-function)))
 
-(defcustom ido-cr+-auto-update-blacklist 'notify
+(defcustom ido-cr+-auto-update-disable-list 'notify
   "Whether to add new overrides when updating ido-cr+.
 
 This variable has 3 possible values, with the following meanings:
 
-  `t': Auto-update the blacklist
+  t: Auto-update the disable list
   `notify': Notify you about updates but do not apply them
-  `nil': Ignore all blacklist updates
+  nil: Ignore all disable list updates
 
-Ido-cr+ comes with a default blacklist for commands that are
-known to be incompatible with ido completion. New versions of
-ido-cr+ may come with updates to this blacklist as more
-incompatible commands are discovered. However, customizing your
-own overrides would normally prevent you from receiving these
-updates, since Emacs will not overwrite your customizations.
+Ido-cr+ comes with a default list of commands that are known to
+be incompatible with ido completion. New versions of ido-cr+ may
+come with updates to this \"disable list\" as more incompatible
+commands are discovered. However, customizing your own overrides
+would normally prevent you from receiving these updates, since
+Emacs will not overwrite your customizations.
 
-To resolve this problem, you can set this variable to `t', and
-then ido-cr+ can automatically add any new built-in overrides
-whenever it is updated. (Actually, the update will happen the
-next time Emacs is restarted after the update.) This allows you
-to add your own overrides but still receive updates to the
-default set.
+To resolve this problem, you can set this variable to t, and then
+ido-cr+ can automatically add any new built-in overrides whenever
+it is updated. (Actually, the update will happen the next time
+Emacs is restarted after the update.) This allows you to add your
+own overrides but still receive updates to the default set.
 
-If you want ido-cr+ to just notify you about new default
-overrides instead of adding them itself, set this variable to
-`notify'. If you don't want this auto-update behavior at all, set
-it to `nil'.
+If you want ido-cr+ to just notify you about new defaults instead
+of adding them itself, set this variable to `notify'. If you
+don't want this auto-update behavior at all, set it to nil.
 
-(Note that having this option enabled effectively prevents you
-from removing any of the built-in default blacklist entries,
-since they will simply be re-added the next time Emacs starts.)"
+\(Note that having this option enabled effectively prevents you
+from removing any of the built-in default entries, since they
+will simply be re-added the next time Emacs starts.)"
   :type '(choice :tag "When new overrides are available:"
                  (const :menu-tag "Auto-add"
                         :tag "Add them automatically"
@@ -1120,24 +1182,32 @@ since they will simply be re-added the next time Emacs starts.)"
                         nil))
   :group 'ido-completing-read-plus)
 
-(defun ido-cr+-update-blacklist (&optional save quiet)
-  "Re-add any missing default blacklist entries.
+(define-obsolete-function-alias
+  'ido-cr+-update-blacklist
+  'ido-cr+-update-disable-list
+  "ido-completing-read+ 4.14")
+
+(defun ido-cr+-update-disable-list (&optional save quiet)
+  "Re-add any missing default entries to `ido-cr+-disable-list'.
 
 This is useful after an update of ido-ubiquitous that adds new
-default overrides. See `ido-cr+-auto-update-blacklist' for more
-information.
+default overrides. See `ido-cr+-auto-update-disable-list' for
+more information.
 
-If SAVE is non-nil, also save the new blacklist to the user's
+If SAVE is non-nil, also save the new disable list to the user's
 Custom file (but only if it was already customized beforehand).
 When called interactively, a prefix argument triggers a save.
 
+Unless QUIET is non-nil, this function produces messages indicating
+all changes that were made.
+
 When called from Lisp code, this function returns non-nil if the
-blacklist was modified."
+disable list was modified."
   (interactive "P")
-  (let* ((var-state (custom-variable-state 'ido-cr+-function-blacklist
-                                           ido-cr+-function-blacklist))
-         (curval ido-cr+-function-blacklist)
-         (defval (eval (car (get 'ido-cr+-function-blacklist 'standard-value))))
+  (let* ((var-state (custom-variable-state 'ido-cr+-disable-list
+                                           ido-cr+-disable-list))
+         (curval ido-cr+-disable-list)
+         (defval (eval (car (get 'ido-cr+-disable-list 'standard-value))))
          (newval (delete-dups (append defval curval)))
          (new-entries (cl-set-difference defval curval :test #'equal))
          (modified nil)
@@ -1146,52 +1216,57 @@ blacklist was modified."
     (cl-case var-state
       (standard
        ;; Var is not customized, just set the new default
-       (ido-cr+--debug-message "Blacklist was not customized, so it has been updated to the new default value.")
-       (setq ido-cr+-function-blacklist defval
+       (ido-cr+--debug-message "Disable list was not customized, so it has been updated to the new default value.")
+       (setq ido-cr+-disable-list defval
              modified new-entries))
       ((saved set changed)
        ;; Var has been customized and saved by the user, so set the
        ;; new value and maybe save it
-       (ido-cr+--debug-message "Updating user-customized blacklist with new default entries.")
-       (setq ido-cr+-function-blacklist newval
+       (ido-cr+--debug-message "Updating user-customized disable list with new default entries.")
+       (setq ido-cr+-disable-list newval
              modified t)
        (when (and save (eq var-state 'saved))
-         (ido-cr+--debug-message "Saving new blacklist value to Custom file.")
-         (customize-save-variable 'ido-cr+-function-blacklist ido-cr+-function-blacklist)
+         (ido-cr+--debug-message "Saving new disable list value to Custom file.")
+         (customize-save-variable 'ido-cr+-disable-list ido-cr+-disable-list)
          (setq saved t)))
       (otherwise
-       (ido-cr+--debug-message "Customization status of blacklist is unknown. Not modifying it.")))
+       (ido-cr+--debug-message "Customization status of disable list is unknown. Not modifying it.")))
     (if (and modified (not quiet))
         (progn
-          (push (format "Added the following entries to `ido-cr+-function-blacklist': %S" new-entries)
+          (push (format "Added the following entries to `ido-cr+-disable-list': %S" new-entries)
                 message-lines)
           (if saved
-              (push "Saved the new value of `ido-cr+-function-blacklist' to your Custom file."
+              (push "Saved the new value of `ido-cr+-disable-list' to your Custom file."
                     message-lines)
-            (push "However, the new value of `ido-cr+-function-blacklist' has not yet been saved for future sessions. To save it. re-run this command with a prefix argument:  `C-u M-x ido-cr+-update-blacklist'; or else manually inspect and save the value using `M-x customize-variable ido-cr+-function-blacklist'."
+            (push "However, the new value of `ido-cr+-disable-list' has not yet been saved for future sessions. To save it. re-run this command with a prefix argument:  `C-u M-x ido-cr+-update-disable-list'; or else manually inspect and save the value using `M-x customize-variable ido-cr+-disable-list'."
                   message-lines)))
-      (push "No updates were required to `ido-cr+-function-blacklist'." message-lines))
+      (push "No updates were required to `ido-cr+-disable-list'." message-lines))
     (unless quiet
       (message (mapconcat #'identity (nreverse message-lines) "\n")))
     modified))
 
-(defun ido-cr+-maybe-update-blacklist ()
-  "Maybe call `ico-cr+-update-blacklist.
+(define-obsolete-function-alias
+  'ido-cr+-maybe-update-blacklist
+  'ido-cr+-maybe-update-disable-list
+  "ido-completing-read+ 4.14")
 
- See `ido-cr+-auto-update-blacklist' for more information."
-  (if ido-cr+-auto-update-blacklist
-      (let* ((curval ido-cr+-function-blacklist)
-             (defval (eval (car (get 'ido-cr+-function-blacklist 'standard-value))))
+(defun ido-cr+-maybe-update-disable-list ()
+  "Maybe call `ico-cr+-update-disable-list.
+
+ See `ido-cr+-auto-update-disable-list' for more information."
+  (if ido-cr+-auto-update-disable-list
+      (let* ((curval ido-cr+-disable-list)
+             (defval (eval (car (get 'ido-cr+-disable-list 'standard-value))))
              (new-entries (cl-set-difference defval curval :test #'equal)))
         (if new-entries
-            (if (eq ido-cr+-auto-update-blacklist 'notify)
-                (display-warning 'ido-completing-read+ (format "There are %s new blacklist entries available. Use `M-x ido-cr+-update-blacklist' to install them. (See `ido-cr+-auto-update-blacklist' for more information.)" (length new-entries)))
-              (ido-cr+--debug-message "Initiating blacklist update.")
-              (ido-cr+-update-blacklist t))
-          (ido-cr+--debug-message "No blacklist updates available.")))
-    (ido-cr+--debug-message "Skipping blacklist update by user request.")))
+            (if (eq ido-cr+-auto-update-disable-list 'notify)
+                (display-warning 'ido-completing-read+ (format "There are %s new disable list entries available. Use `M-x ido-cr+-update-disable-list' to install them. (See `ido-cr+-auto-update-disable-list' for more information.)" (length new-entries)))
+              (ido-cr+--debug-message "Initiating disable list update.")
+              (ido-cr+-update-disable-list t))
+          (ido-cr+--debug-message "No disable list updates available.")))
+    (ido-cr+--debug-message "Skipping disable list update by user request.")))
 
-(ido-cr+-maybe-update-blacklist)
+(ido-cr+-maybe-update-disable-list)
 
 (provide 'ido-completing-read+)
 
